/*************************************************************************
* (C) 2013 AITNET ltd - Sofia/Bulgaria - <misho@aitnet.org>
*  by Michael Pounov <misho@elwix.org>
*
* $Author: misho $
* $Id: anet.h,v 1.6 2013/07/09 00:18:19 misho Exp $
*
**************************************************************************
The ELWIX and AITNET software is distributed under the following
terms:

All of the documentation and software included in the ELWIX and AITNET
Releases is copyrighted by ELWIX - Sofia/Bulgaria <info@elwix.org>

Copyright 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
	by Michael Pounov <misho@elwix.org>.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
This product includes software developed by Michael Pounov <misho@elwix.org>
ELWIX - Embedded LightWeight unIX and its contributors.
4. Neither the name of AITNET nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY AITNET AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
#ifndef __ANET_H
#define __ANET_H


struct e_ether_addr {
	u_int8_t ether_addr_octet[6];
};
typedef struct e_ether_addr ether_addr_t;

typedef union {
	struct sockaddr_storage	ss;
	struct sockaddr		sa;
	struct sockaddr_un	sun;
	struct sockaddr_in	sin;
	struct sockaddr_in6	sin6;
	struct sockaddr_dl	sdl;
} sockaddr_t;
#define E_SOCKADDR_INIT	{ .ss = { 0 } }

typedef union {
	union {
		in_addr_t	s_addr;
		u_char		s4_addr[sizeof(in_addr_t)];
	}		in;
	struct in6_addr	in6;
} inaddr_t;

typedef struct {
	sockaddr_t	addr;
	inaddr_t	mask;
} netaddr_t;
#define E_CIDRMASK(x)	htonl((((1 << (x)) - 1) << (32 - (x))) & 0xFFFFFFFF)


/*
 * e_usleep() - usleep() replacement for ELWIX
 *
 * @usec = microseconds for sleep
 * return: -1 interrupted by signal or 0 ok
 */
int e_usleep(unsigned int usec);
/*
 * e_link_ntoa() - String ethernet address from link address
 *
 * @sdl = link address
 * return: =NULL error or !=NULL ethernet address, should be e_free()
 */
char *e_link_ntoa(const struct sockaddr_dl *sdl);
/*
 * e_ether_ntoa() - Convert ethernet address to string
 *
 * @n = ethernet address structure, like struct ether_addr
 * @a = string
 * @len = string length
 * return: NULL error or !=NULL string a
 */
char *e_ether_ntoa(const ether_addr_t * __restrict n, char * __restrict a, int len);
/*
 * e_ether_aton() - Convert string to ethernet address
 *
 * @a = string
 * @e = ethernet address structure, like struct ether_addr
 * return: NULL error or !=NULL ethernet address structure
 */
ether_addr_t *e_ether_aton(const char *a, ether_addr_t * __restrict e);
/*
 * e_n2port() - Extract port from network structure
 *
 * @addr = Address
 * return: 0 not supported family type or port number
 */
unsigned short e_n2port(sockaddr_t * __restrict addr);
/*
 * e_n2addr() - Extract address from network structure
 *
 * @addr = Address
 * @val = Value for store string address
 * return: NULL error or !=NULL string address from val
 */
const char *e_n2addr(sockaddr_t * __restrict addr, ait_val_t * __restrict val);
/*
 * e_gethostbyname() - Get host and port and make network structure
 *
 * @psHost = Hostname
 * @port = Port
 * @addr = Network address structure
 * return: NULL error or !=NULL network structure
 */
sockaddr_t *e_gethostbyname(const char *psHost, unsigned short port, 
		sockaddr_t * __restrict addr);
/*
 * e_addrcmp() - Compare network addresses
 *
 * @a = 1st address
 * @b = 2nd address
 * @p = compare and ports, if family is AF_INET or AF_INET6
 * return: 0 is equal or !=0 is different
 */
int e_addrcmp(sockaddr_t * __restrict a, sockaddr_t * __restrict b, int p);
/*
 * e_innet() - Test address match in network
 *
 * @net = network
 * @addr = address
 * return: -1 error, 0 match or 1 not match
 */
int e_innet(netaddr_t * __restrict net, inaddr_t * __restrict addr);
/*
 * e_getnet() - Get network from string
 *
 * @net = Network string (format: <net[/cidr]>)
 * return: NULL error or !=NULL network should be e_free()
 */
netaddr_t *e_getnet(const char *net);
/*
 * e_ether_addr() - Get or set ethernet address from interface name
 *
 * @ifname = interface name
 * @addr = if addr is !=NULL then set this for new address
 * return: NULL error or !=NULL get ethernet address
 */
ether_addr_t *e_ether_addr(const char *ifname, ether_addr_t * __restrict addr);


#endif
